/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or any later
 * version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

package org.objectweb.jonas.discovery;

/**
 * @author Takoua Abdellatif
 */
import javax.management.JMException;
import javax.management.MBeanRegistration;
import javax.management.MBeanServer;
import javax.management.MBeanServerNotification;
import javax.management.Notification;
import javax.management.NotificationListener;
import javax.management.ObjectName;
import javax.management.remote.JMXServiceURL;

import org.objectweb.jonas.jmx.J2eeObjectName;

/**
 * DiscoveryManager goal is to listen to a well known and reconfigurable
 * IpAddress and to give appropriate JMXURL to manage remotely Jonas server.
 *
 * @author <a href="mailto:Takoua.Abdellatif@inria.fr">Takoua Abdellatif </a>
 * @version 1.0
 */
public class DiscoveryManager implements
    DiscoveryManagerMBean,
    MBeanRegistration,
    NotificationListener {

    /**
     * <code>listeningPort</code> for multicast socket creation
     */
    private int           listeningPort;
    /**
     * <code>listeningIp</code> for multicast socket creation
     */
    private String        listeningIp;
    /**
     * Runnable object that implements multicast communication with
     * the servers in the management domain
     */
    private DiscoveryComm dc;
    /**
     * Thread associated to dc
     */
    private Thread        commDaemon;
    /**
     * Current MBean Server
     */
    private MBeanServer   mbeanServer;
    private ObjectName    myOn;
    private int           ttl = 1;

    private String jonasName = null;
    private String domainName = null;
    private String[] urls = null;

    /**
     *
     */
    public DiscoveryManager(int listeningPort, String listeningIp) {
      this.listeningIp = listeningIp;
      this.listeningPort = listeningPort;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#start()
     */

    public void start() {

        // creates an instance of the Discovery communication class
        dc = new DiscoveryComm(this);
        dc.setJonasName(jonasName);
        dc.setDomainName(domainName);
        dc.setUrls(urls);

        if (commDaemon == null)
            commDaemon = new Thread(dc, "commDaemon");
        commDaemon.start();
    }

    /**
     *
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#getListeningPort()
     */
    public int getListeningPort() {
        return listeningPort;
    }

    /**
     * (non-Javadoc)
     *
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#setListeningPort()
     */
    public void setListeningPort(int listeningPort) {
        this.listeningPort = listeningPort;

    }

    /**
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#getListeningIp()
     */
    public String getListeningIp() {
        return listeningIp;
    }

    /**
     *
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#setListeningIp()
     */
    public void setListeningIp(String listeningIp) {
        this.listeningIp = listeningIp;

    }

    /**
     *
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#setTimeToLive()
     */
    public void setTimeToLive(int ttl) {
        this.ttl = ttl;
    }

    /**
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#getTimeToLive()
     */
    public int getTimeToLive() {
        return this.ttl;
    }

    /**
     * @see javax.management.MBeanRegistration#preRegister(javax.management.MBeanServer,
     *      javax.management.ObjectName)
     */
    public ObjectName preRegister(MBeanServer mbeanServer, ObjectName on) throws Exception {
        // set mbeanServer
        this.mbeanServer = mbeanServer;
        this.myOn = on;

        return myOn;
    }

    /**
     *
     * @see javax.management.MBeanRegistration#postRegister(java.lang.Boolean)
     */
    public void postRegister(Boolean arg0) {
      // Add discovery manager as a listener of the delegate object to receive unregistration notification of the j2EEServer.
      ObjectName delegate;
      try {
        delegate = new ObjectName("JMImplementation:type=MBeanServerDelegate");
        mbeanServer.addNotificationListener(delegate, this, null, null);
      } catch (JMException e) {
        e.printStackTrace();
      }
    }

    /**
     * @see javax.management.MBeanRegistration#preDeregister()
     */
    public void preDeregister() throws Exception {
        // stop the communication Daemon
        dc.stop();
    }

    /**
     * @see javax.management.MBeanRegistration#postDeregister()
     */
    public void postDeregister() {

    }

    /**
     *
     * @see javax.management.NotificationListener#handleNotification(javax.management.Notification,
     *      java.lang.Object)
     */
    public void handleNotification(Notification notification, Object handback) {
        if (notification.getType().equals(MBeanServerNotification.UNREGISTRATION_NOTIFICATION)) {
            ObjectName notificationSender = ((MBeanServerNotification) notification).getMBeanName();
            ObjectName j2eeServerName = J2eeObjectName.J2EEServer(domainName, jonasName);
            if (notificationSender.equals(j2eeServerName)) {
                dc.stop();
            }
        }
    }

    /**
     * @param jonasName The jonasName to set.
     */
    protected void setJonasName(String jonasName) {
        this.jonasName = jonasName;
    }
    /**
     * @param domainNamle The domainNamle to set.
     */
    protected void setDomainName(String domainNamle) {
        this.domainName = domainNamle;
    }
    /**
     * @param urls The urls to set.
     */
    protected void setUrls(String[] urls) {
        this.urls = urls;
    }

    /**
     * Stop 
     */
    public void stop() {
        dc.stop();
    }
}