/**
* JOnAS: Java(TM) Open Application Server
* Copyright (C) 2004 Bull S.A.
* Contact: jonas-team@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: DiscoveryServiceImpl.java,v 1.4.2.1 2005/02/18 10:34:48 danesa Exp $
* --------------------------------------------------------------------------
*/

package org.objectweb.jonas.discovery;

import java.util.ArrayList;

import javax.management.JMException;
import javax.management.MBeanServer;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;
import javax.naming.NamingException;

import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 *
 * @author Adriana Danes
 *
 * The discovery service creates and starts at least a DiscoveryManager which is a MBean
 * that multicasts <code>discovery information</code> to all the servers who joined the
 * <code>discovery multicast group</code>.
 * <p>
 * The discovery service may also create a Enroller and a DiscoveryClient, and in this case
 * the current server becomes a <code>discovery server</code>.
 * <p>
 * <code>Discovery information</code> contains information allowing to remotely manage a server in the group.
 * <p>
 * <code>Discovery multicast group</code> is a group of servers which can be managed remotely by a discovery server.
 * <p>
 * <code>Discovery server</code> is the server in the group who detains the discovery information concerning all the
 * servers in the group.
 */
public class DiscoveryServiceImpl extends AbsServiceImpl implements DiscoveryService , DiscoveryServiceImplMBean {

    private final String DISCOVERY_SOURCE_PORT_DEFAULT = "9888";
    private String listeningIp = null;
    private int listeningPort;
    private int sourcePort;
    /**
     * set to true if enroller and discoveryClient started
     */
    private boolean isDiscoveryMaster = false;
    /**
     * Reference to a MBean server.
     */
    private MBeanServer mbeanServer = null;

    /**
     * jmx service reference
     */
    private JmxService jmxService = null;
    /**
     * Logger for this service.
     */
    private static Logger logger = null;

    /**
     * @return the multicast group IP address used by the discovery service
     */
    public String getMulticastAddress() {
        return listeningIp;
    }

    /**
     * @return the multicast group port number used by the discovery service
     */
    public Integer getMulticastPort() {
        return new Integer(listeningPort);
    }

    /**
     * @return true if the current server is a discovery server
     */
    public Boolean isDiscoveryMaster() {
        return new Boolean(isDiscoveryMaster);
    }
    /**
     * Management operation allowing to make the current server become a
     * master if its not already.
     * @throws JMException a JMX exception occured when trying to make current server a discovery master
     */
    public void startDiscoveryMaster() throws JMException {
        if (!isDiscoveryMaster) {
            createEnroller();
            createDiscClient();
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.jonas.service.AbsServiceImpl#doInit(javax.naming.Context)
     */
    protected void doInit(Context ctx) throws ServiceException {
        //Init the logger
        logger = Log.getLogger(Log.JONAS_DISCOVERY_PREFIX);

        // Get service configuration
        try {
            listeningIp = (String) ctx.lookup("jonas.service.discovery.multicast.address");
            String sListeningPort = (String) ctx.lookup("jonas.service.discovery.multicast.port");
            listeningPort = (Integer.valueOf(sListeningPort)).intValue();
        } catch (NamingException ne) {
            String err = "Cannot read initializations arguments in service context";
            logger.log(BasicLevel.ERROR, err);
            throw new ServiceException(err, ne);
        }
        // Get info allowing to see if this is a discovery master
        try {
            String sMaster = (String) ctx.lookup("jonas.service.discovery.master");
            if (sMaster != null && sMaster.equals("true")) {
                isDiscoveryMaster = true;
            }
        } catch (NamingException ne) {
            // isDiscoveryMaster rests false
        }
        // If configuration does not says this ia a master, allow
        // however to start it as a master is the server name is
        // identical to the domain name (in the default case this
        // is "jonas", "jonas")
        if (!isDiscoveryMaster) {
            if (getDomainName().equals(getJonasServerName())) {
                isDiscoveryMaster = true;
            }
        }

        String sSourcePort = null;
        try {
            sSourcePort = (String) ctx.lookup("jonas.service.discovery.source.port");
        } catch (NamingException ne) {
            sSourcePort = DISCOVERY_SOURCE_PORT_DEFAULT;
        }
        sourcePort = (Integer.valueOf(sSourcePort)).intValue();

        // Get JOnAS references allowing to start execution
        ServiceManager sm  = null;
        try {
            sm = ServiceManager.getInstance();
        } catch (Exception e) {
            String err = "Cannot get ServiceManager instance";
            logger.log(BasicLevel.ERROR, err);
            throw new ServiceException(err, e);
        }
        jmxService = ((JmxService) sm.getJmxService());
        try {
            mbeanServer = jmxService.getJmxServer();
        } catch (ServiceException e) {
            // the JMX service may not be started
            mbeanServer = null;
            String err = "Cannot get MBeanServer reference";
            logger.log(BasicLevel.ERROR, err);
            throw new ServiceException(err, e);
        }
    }

    /**
     * Start the discovery service
     * @throws ServiceException An error occured when starting the service
     */
    protected void doStart() throws ServiceException {
        // Create discovery manager
        DiscoveryManager dm = new DiscoveryManager(listeningPort, listeningIp);
        dm.setDomainName(jmxService.getDomainName());
        dm.setJonasName(jmxService.getJonasServerName());
        JMXServiceURL[] connectorServerURLs = jmxService.getConnectorServerURLs();
        ArrayList urlsList = new ArrayList();
        for (int i = 0; i < connectorServerURLs.length; i++) {
            // The connectorServerURLs may contain null
            // if the list of protocols in Carol contain
            // other protocols than the standard ones (JRMP, JEREMIE, IIOP, CMI)
            if (connectorServerURLs[i] != null) {
                urlsList.add(connectorServerURLs[i].toString());
            }
        }
        String[] urls = new String[urlsList.size()];
        for (int i = 0; i < urls.length; i++) {
            urls[i] = (String) urlsList.get(i);
        }
        dm.setUrls(urls);
        try {
            // Register  DiscoveryManager MBean
            if (mbeanServer != null) {
                mbeanServer.registerMBean(dm, JonasObjectName.discoveryManager());
            }
        } catch (JMException e) {
            throw new ServiceException("Problem when starting the Discovery Service: ", e);
        }
        // Start discovery manager
        dm.start();

        if (isDiscoveryMaster) {
            // Create enroller
            try {
                createEnroller();
            } catch (JMException e) {
                throw new ServiceException("Problem when starting the Discovery Service: ", e);
            }

            // Create discovery client
            try {
                createDiscClient();
            } catch (JMException e) {
                throw new ServiceException("Problem when starting the Discovery Service: ", e);
            }
        }

        // Create and register the service MBean
        try {
            if (mbeanServer != null) {
                mbeanServer.registerMBean(this, JonasObjectName.discoveryService());
            }
        } catch (JMException e) {
            throw new ServiceException("Problem when starting the Discovery Service: ", e);
        }
    }
    
    private void createEnroller() throws JMException {
        Enroller enroller = new Enroller(listeningPort, listeningIp);
        if (mbeanServer != null) {
            mbeanServer.registerMBean(enroller, JonasObjectName.discoveryEnroller());
        }
    }

    private void createDiscClient() throws JMException {
        DiscoveryClient dc = new DiscoveryClient(listeningPort, listeningIp, sourcePort);
        // Register MBean
        if (mbeanServer != null) {
            mbeanServer.registerMBean(dc, JonasObjectName.discoveryClient());
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.jonas.service.AbsServiceImpl#doStop()
     */
    protected void doStop() throws ServiceException {
        // TO DO
    	// ....

        // Unegister the service MBean
        try {
        	if (mbeanServer != null) {
        		mbeanServer.registerMBean(this, JonasObjectName.discoveryService());
        	}
        } catch (JMException e) {
            throw new ServiceException("Problem when stoping the Discovery Service: ", e);
        }
    }
}